import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tim_fluttify/src/android/android.export.g.dart';
import 'package:tim_fluttify/src/android/com/tencent/imsdk/v2/V2TIMSDKConfig.g.dart';
import 'package:tim_fluttify/src/ios/V2TIMSDKConfig.g.dart';
import 'package:tim_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';
import 'extensions.dart';

part 'callbacks.dart';
part 'models.dart';

class Tim {
  static Tim instance = Tim._();

  Tim._();

  /// 初始化
  Future<void> init(
    int appId, {
    LogLevel logLevel = LogLevel.info,
  }) async {
    await platform(
      android: (pool) async {
        final config = await com_tencent_imsdk_v2_V2TIMSDKConfig.create__();
        await config.setLogLevel(logLevel.androidLevel());

        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();
        final context = await android_app_Application.get();

        await manager.initSDK(context, appId, config, _AndroidTimSDKCallback());

        pool..add(config)..add(manager)..add(context);
      },
      ios: (pool) async {
        final config = await V2TIMSDKConfig.create__();
        await config.set_logLevel(logLevel.iosLevel());

        final manager = await V2TIMManager.sharedInstance();

        await manager.initSDK_config_listener(
          appId,
          config,
          _IOSTimSDKCallback(),
        );

        pool..add(config)..add(manager);
      },
    );
  }

  /// 登录
  Future<void> login(String userId, String userSig) async {
    final completer = Completer();

    await platform(
      android: (pool) async {
        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();
        await manager.login(
            userId, userSig, _AndroidTimVoidCallback(completer));

        pool.add(manager);
      },
      ios: (pool) async {
        final manager = await V2TIMManager.sharedInstance();
        await manager.login_userSig_succ_fail(
          userId,
          userSig,
          completer.complete,
          (code, desc) => completer.completeError('$code $desc'),
        );

        pool.add(manager);
      },
    );

    return completer.future;
  }

  /// 登出
  Future<void> logout() async {
    final completer = Completer.sync();

    platform(
      android: (pool) async {
        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();
        manager.logout(_AndroidTimVoidCallback(completer));

        pool.add(manager);
      },
      ios: (pool) async {
        final manager = await V2TIMManager.sharedInstance();
        manager.logout_fail(
          completer.complete,
          (code, desc) => completer.completeError('$code $desc'),
        );

        pool.add(manager);
      },
    );

    return completer.future;
  }

  /// 发送单聊文字消息
  Future<Message> sendTextMessage(
    String sessionId,
    String text, {
    @required SessionType type,
  }) async {
    return platform(
      android: (pool) async {
        final completer = Completer<com_tencent_imsdk_v2_V2TIMMessage>.sync();

        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();

        switch (type) {
          case SessionType.c2c:
            await manager.sendC2CTextMessage(
              text,
              sessionId,
              _AndroidTimValueCallback(completer),
            );
            break;
          case SessionType.group:
            await manager.sendGroupTextMessage(
              text,
              sessionId,
              0,
              _AndroidTimValueCallback(completer),
            );
            break;
        }

        pool.add(manager);
        return completer.future.then((value) => Message.android(value));
      },
      ios: (pool) async {
        final completer = Completer<V2TIMMessage>.sync();

        final manager = await V2TIMManager.sharedInstance();
        switch (type) {
          case SessionType.c2c:
            manager.sendC2CTextMessage_to_succ_fail(
              text,
              sessionId,
              completer.complete,
              (code, desc) => completer.completeError('$code $desc'),
            );
            break;
          case SessionType.group:
            manager.sendGroupTextMessage_to_priority_succ_fail(
              text,
              sessionId,
              V2TIMMessagePriority.V2TIM_PRIORITY_DEFAULT,
              completer.complete,
              (code, desc) => completer.completeError('$code $desc'),
            );
            break;
        }

        pool.add(manager);
        return completer.future.then((value) => Message.ios(value));
      },
    );
  }

  /// 获取所有的会话
  ///
  /// 从索引[nextSeq]开始, 获取[count]条会话
  Future<List<Conversation>> fetchConversationList({
    int nextSeq = 0,
    int count = 20,
  }) async {
    return platform(
      android: (pool) async {
        final completer =
            Completer<com_tencent_imsdk_v2_V2TIMConversationResult>.sync();

        final manager =
            await com_tencent_imsdk_v2_V2TIMManager.getConversationManager();
        await manager.getConversationList(
          nextSeq,
          count,
          _AndroidTimValueCallback(completer),
        );

        final conversationResult = await completer.future;
        final conversationList = await conversationResult.getConversationList();

        pool..add(manager)..add(conversationResult);
        return conversationList.map((e) => Conversation.android(e)).toList();
      },
      ios: (pool) async {
        final completer = Completer<List<V2TIMConversation>>.sync();

        final manager = await V2TIMManager.sharedInstance();
        await manager.getConversationList_count_succ_fail(
          nextSeq,
          count,
          (list, nextSeq, isFinished) => completer.complete(list),
          (code, desc) => completer.completeError('$code $desc'),
        );

        pool.add(manager);
        return completer.future
            .then((e) => e.map((e) => Conversation.ios(e)).toList());
      },
    );
  }

  /// 获取个人资料
  Future<UserInfo> fetchMyProfile() async {
    return platform(
      android: (pool) async {
        final completer =
            Completer<List<com_tencent_imsdk_v2_V2TIMUserFullInfo>>.sync();

        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();
        final loginUser = await manager.getLoginUser();
        if (loginUser != null) {
          manager
              .getUsersInfo([loginUser], _AndroidTimValueCallback(completer));

          pool.add(manager);
          return completer.future
              .then((value) => value[0])
              .then((value) => UserInfo.android(value));
        } else {
          pool.add(manager);
          return Future.error('用户未登录');
        }
      },
      ios: (pool) async {
        final completer = Completer<List<V2TIMUserFullInfo>>.sync();

        final manager = await V2TIMManager.sharedInstance();
        final loginUser = await manager.getLoginUser();
        if (loginUser != null) {
          manager.getUsersInfo_succ_fail(
            [loginUser],
            (infoList) => completer.complete(infoList),
            (code, desc) => completer.completeError('$code $desc'),
          );

          pool.add(manager);
          return completer.future
              .then((value) => value[0])
              .then((value) => UserInfo.ios(value));
        } else {
          pool.add(manager);
          return Future.error('用户未登录');
        }
      },
    );
  }

  /// 获取用户资料
  Future<List<UserInfo>> fetchUserProfile(List<String> userIdList) async {
    return platform(
      android: (pool) async {
        final completer =
            Completer<List<com_tencent_imsdk_v2_V2TIMUserFullInfo>>.sync();

        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();
        manager.getUsersInfo(userIdList, _AndroidTimValueCallback(completer));

        pool.add(manager);
        return completer.future
            .then((value) => value.map((e) => UserInfo.android(e)).toList());
      },
      ios: (pool) async {
        final completer = Completer<List<V2TIMUserFullInfo>>.sync();

        final manager = await V2TIMManager.sharedInstance();
        manager.getUsersInfo_succ_fail(
          userIdList,
          (infoList) => completer.complete(infoList),
          (code, desc) => completer.completeError('$code $desc'),
        );

        pool.add(manager);
        return completer.future
            .then((value) => value.map((e) => UserInfo.ios(e)).toList());
      },
    );
  }

  /// 获取会话历史消息列表
  Future<List<Message>> fetchHistoryMessage(
    String sessionId, {
    @required SessionType type,
    int count = 20,
    Message lastMessage,
  }) async {
    return platform(
      android: (pool) async {
        final completer =
            Completer<List<com_tencent_imsdk_v2_V2TIMMessage>>.sync();

        final manager =
            await com_tencent_imsdk_v2_V2TIMManager.getMessageManager();
        switch (type) {
          case SessionType.c2c:
            await manager.getC2CHistoryMessageList(
              sessionId,
              count,
              lastMessage._androidModel,
              _AndroidTimValueCallback(completer),
            );
            break;
          case SessionType.group:
            await manager.getGroupHistoryMessageList(
              sessionId,
              count,
              lastMessage._androidModel,
              _AndroidTimValueCallback(completer),
            );
            break;
        }

        pool.add(manager);
        return completer.future
            .then((value) => value.map((e) => Message.android(e)).toList());
      },
      ios: (pool) async {
        final completer = Completer<List<V2TIMMessage>>.sync();

        final manager = await V2TIMManager.sharedInstance();
        switch (type) {
          case SessionType.c2c:
            await manager.getC2CHistoryMessageList_count_lastMsg_succ_fail(
              sessionId,
              count,
              lastMessage._iosModel,
              (msgs) => completer.complete(msgs),
              (code, desc) => completer.completeError('$code $desc'),
            );
            break;
          case SessionType.group:
            await manager.getGroupHistoryMessageList_count_lastMsg_succ_fail(
              sessionId,
              count,
              lastMessage._iosModel,
              (msgs) => completer.complete(msgs),
              (code, desc) => completer.completeError('$code $desc'),
            );
            break;
        }

        pool.add(manager);
        return completer.future
            .then((value) => value.map((e) => Message.ios(e)).toList());
      },
    );
  }

  /// 获取当前登录用户
  Future<String> fetchLoginUser() async {
    return platform(
      android: (pool) async {
        final manager = await com_tencent_imsdk_v2_V2TIMManager.getInstance();

        pool.add(manager);
        return manager.getLoginUser();
      },
      ios: (pool) async {
        final manager = await V2TIMManager.sharedInstance();

        pool.add(manager);
        return manager.getLoginUser();
      },
    );
  }

  /// 获取好友列表
  Future<List<Friend>> fetchFriendList() async {
    return platform(
      android: (pool) async {
        final completer =
            Completer<List<com_tencent_imsdk_v2_V2TIMFriendInfo>>.sync();

        final manager =
            await com_tencent_imsdk_v2_V2TIMManager.getFriendshipManager();

        await manager.getFriendList(_AndroidTimValueCallback(completer));
        pool.add(manager);
        return completer.future
            .then((value) => value.map((e) => Friend.android(e)).toList());
      },
      ios: (pool) async {
        final completer = Completer<List<V2TIMFriendInfo>>.sync();

        final manager = await V2TIMManager.sharedInstance();

        await manager.getFriendList_fail(
          (infoList) => completer.complete(infoList),
          (code, desc) => completer.completeError('$code $desc'),
        );
        pool.add(manager);
        return completer.future
            .then((value) => value.map((e) => Friend.ios(e)).toList());
      },
    );
  }

  /// 标记已读
  Future<void> markMessageAsRead(
    String sessionId, {
    @required SessionType type,
  }) async {
    return platform(
      android: (pool) async {
        final completer = Completer<void>.sync();

        final manager =
            await com_tencent_imsdk_v2_V2TIMManager.getMessageManager();

        switch (type) {
          case SessionType.c2c:
            await manager.markC2CMessageAsRead(
              sessionId,
              _AndroidTimVoidCallback(completer),
            );
            break;
          case SessionType.group:
            await manager.markGroupMessageAsRead(
              sessionId,
              _AndroidTimVoidCallback(completer),
            );
            break;
        }

        pool.add(manager);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<void>.sync();

        final manager = await V2TIMManager.sharedInstance();

        switch (type) {
          case SessionType.c2c:
            await manager.markC2CMessageAsRead_succ_fail(
              sessionId,
              () => completer.complete(),
              (code, desc) => completer.completeError('$code $desc'),
            );
            break;
          case SessionType.group:
            await manager.markGroupMessageAsRead_succ_fail(
              sessionId,
              () => completer.complete(),
              (code, desc) => completer.completeError('$code $desc'),
            );
            break;
        }
        pool.add(manager);
        return completer.future;
      },
    );
  }

  /// 添加好友
  Future<AddFriendResult> addFriend(String userId) async {
    return platform(
      android: (pool) async {
        final completer =
            Completer<com_tencent_imsdk_v2_V2TIMFriendOperationResult>.sync();

        final manager =
            await com_tencent_imsdk_v2_V2TIMManager.getFriendshipManager();

        final application = await com_tencent_imsdk_v2_V2TIMFriendAddApplication
            .create__String(userId);
        await manager.addFriend(
          application,
          _AndroidTimValueCallback(completer),
        );

        pool..add(manager)..add(application);
        return completer.future.then((value) => AddFriendResult.android(value));
      },
      ios: (pool) async {
        final completer = Completer<V2TIMFriendOperationResult>.sync();

        final manager = await V2TIMManager.sharedInstance();

        final application = await V2TIMFriendAddApplication.create__();
        await application.set_userID(userId);

        await manager.addFriend_succ_fail(
          application,
          completer.complete,
          (code, desc) => completer.completeError('$code $desc'),
        );

        pool..add(manager)..add(application);
        return completer.future.then((value) => AddFriendResult.ios(value));
      },
    );
  }
}
